"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Codemirror = void 0;
const react_1 = __importStar(require("react"));
const classnames_1 = __importDefault(require("classnames"));
const lodash_isequal_1 = __importDefault(require("lodash.isequal"));
const codemirror_1 = __importDefault(require("codemirror"));
const Codemirror = ({ autoFocus, className, codeMirrorInstance = codemirror_1.default, name, path, defaultValue, options, preserveScrollPosition = false, value, children, onChange, onCursorActivity, onFocusChange, onScroll, }) => {
    const codemirrorInstance = react_1.useRef(null);
    const [isFocused, setFocused] = react_1.useState(false);
    const textareaNode = react_1.useRef(null);
    const editorClassName = classnames_1.default('ReactCodeMirror', isFocused ? 'ReactCodeMirror--focused' : null, className);
    function setOptionIfChanged(optionName, newValue) {
        const codeMirror = codemirrorInstance.current;
        if (!codeMirror) {
            return;
        }
        const oldValue = codeMirror.getOption(optionName);
        if (!lodash_isequal_1.default(oldValue, newValue)) {
            codeMirror.setOption(optionName, newValue);
        }
    }
    function focusChanged(focused, instance, event) {
        setFocused(focused);
        onFocusChange && onFocusChange(focused, instance, event);
    }
    function cursorActivity(cm) {
        onCursorActivity && onCursorActivity(cm);
    }
    function scrollChanged(cm) {
        onScroll && onScroll(cm.getScrollInfo());
    }
    function codemirrorValueChanged(instance, changeObj) {
        // @ts-ignore
        if (onChange && instance.origin !== 'setValue') {
            onChange(instance.getValue(), changeObj);
        }
    }
    react_1.useEffect(() => {
        if (textareaNode) {
            codemirrorInstance.current = codeMirrorInstance.fromTextArea(textareaNode, options);
            const codeMirror = codemirrorInstance.current;
            if (!codeMirror) {
                return;
            }
            codeMirror.on('change', codemirrorValueChanged);
            codeMirror.on('cursorActivity', cursorActivity);
            codeMirror.on('focus', focusChanged.bind(null, true));
            codeMirror.on('blur', focusChanged.bind(null, false));
            codeMirror.on('scroll', scrollChanged);
            codeMirror.setValue(defaultValue || value || '');
        }
        return () => {
            const codeMirror = codemirrorInstance.current;
            if (!codeMirror) {
                return;
            }
            if (codeMirror) {
                codeMirror.toTextArea();
            }
        };
    }, [textareaNode, defaultValue, value]);
    react_1.useEffect(function () {
        const codeMirror = codemirrorInstance.current;
        if (!codeMirror) {
            return;
        }
        if (preserveScrollPosition) {
            const prevScrollPosition = codeMirror.getScrollInfo();
            codeMirror.setValue(value);
            codeMirror.scrollTo(prevScrollPosition.left, prevScrollPosition.top);
        }
        else {
            codeMirror.setValue(value);
        }
        if (typeof options === 'object') {
            Object.entries(options).forEach(([optionName, optionValue]) => 
            // @ts-ignore
            setOptionIfChanged(optionName, optionValue));
        }
    }, [value, JSON.stringify(options)]);
    return (react_1.default.createElement("div", { className: editorClassName },
        react_1.default.createElement("textarea", { ref: (ref) => {
                textareaNode.current = ref;
            }, name: name || path, defaultValue: value, autoComplete: "off", autoFocus: autoFocus })));
};
exports.Codemirror = Codemirror;
